/*
The NIST RCS (Real-time Control Systems)
 library is public domain software, however it is preferred
 that the following disclaimers be attached.
 
Software Copywrite/Warranty Disclaimer
 
   This software was developed at the National Institute of Standards and
Technology by employees of the Federal Government in the course of their
official duties. Pursuant to title 17 Section 105 of the United States
Code this software is not subject to copyright protection and is in the
public domain. NIST Real-Time Control System software is an experimental
system. NIST assumes no responsibility whatsoever for its use by other
parties, and makes no guarantees, expressed or implied, about its
quality, reliability, or any other characteristic. We would appreciate
acknowledgement if the software is used. This software can be
redistributed and/or modified freely provided that any derivative works
bear some notice that they are derived from it, and any modified
versions bear some notice that they have been modified.
 
 
 
 */

package diagapplet.plotter;

import java.util.Vector;
import java.awt.Color;



/**
 * Class containing all of the data for a single plot.
 * @author Will Shackleford <shackle@nist.gov>
 */
public class PlotData
{
	private static int count=0;
	public final int num = ++count;
    private Vector<PlotPoint> v = null;
    private PlotPoint plot_point_array[] = null;
    Color line_color = null;
    Color point_color = null;
    public String name = "";
    public String short_name="";
    public String add_to_short_key="";
    public String add_to_key="";
    
    public double integral=0;
    public double stddev=0;
    public double derivmean=0;
    public double mean=0;
    public int point_count = 0;
    public double last_x;
    public double last_y;
    public int last_compare_index=0;
    public boolean array_type=false;
    public boolean delete_me=false;
    
    String line_style = null;
    int line_width;
    int coordinate_type;
    public boolean mark_points = true;
    public int current_size=0;
    public int v_offset = 0;
    public int p2_count = 0;
    public String heading_string=null;
    public boolean show[] = null;
    public boolean painting=false;
    public PlotPoint last_pp_set=null;
    public int last_pp_set_index=-1;
    public PlotData y_plot_data=null;
    public boolean no_key=false;
    public boolean print_v=false;
    public boolean is_y_plot=false;
    
    public double max_x = Double.NEGATIVE_INFINITY;
    public double min_x = Double.POSITIVE_INFINITY;
    public double max_y = Double.NEGATIVE_INFINITY;
    public double min_y = Double.POSITIVE_INFINITY;
    
    public double max_orig_x = Double.NEGATIVE_INFINITY;
    public double min_orig_x = Double.POSITIVE_INFINITY;
    public double max_orig_y = Double.NEGATIVE_INFINITY;
    public double min_orig_y = Double.POSITIVE_INFINITY;
    
    public double max_pre_f_x = Double.NEGATIVE_INFINITY;
    public double min_pre_f_x = Double.POSITIVE_INFINITY;
    public double max_pre_f_y = Double.NEGATIVE_INFINITY;
    public double min_pre_f_y = Double.POSITIVE_INFINITY;
    
    public double min_time = Double.POSITIVE_INFINITY;
    public double max_time = Double.NEGATIVE_INFINITY;
    
    static public int fixed_size =-1;
    /*
      public static void dumpAllStackTraces()
      {
      Iterator i = Thread.getAllStackTraces().values().iterator();
      while(i.hasNext())
	    {
	    StackTraceElement ste[] = (StackTraceElement[]) i.next();
	    System.out.println("-----------------------------");
	    for(int ii = 0; ii < ste.length; ii++)
	    {
	    System.out.println("\tste["+ii+"]="+ste[ii]);
	    }
	    System.out.println("");
	    }
      }
     */
    
    public int get_num_points()
    {
        if(fixed_size > 0)
        {
            return fixed_size;
        }
	if(current_size > v.size())
	{
	    current_size = v.size();
	}
	return current_size;
    }
    
    public PlotData()
    {
        if(fixed_size > 0)
        {
            plot_point_array = new PlotPoint[fixed_size];
            current_size=fixed_size;
        }
        else
        {
            v = new Vector<PlotPoint>();
        }
	show = new boolean[16];
	for(int i = 0; i < show.length; i++)
	{
	    show[i] = true;
	}
    }
    
    public void CheckPoint(PlotPoint pp)
    {
	if(max_orig_x < pp.orig_x)
	{
	    max_orig_x = pp.orig_x;
	}
	if(min_orig_x > pp.orig_x)
	{
	    min_orig_x = pp.orig_x;
	}
	if(max_orig_y < pp.orig_y)
	{
	    max_orig_y = pp.orig_y;
	}
	if(min_orig_y > pp.orig_y)
	{
	    min_orig_y = pp.orig_y;
	}
	if(max_pre_f_x < pp.pre_f_x)
	{
	    max_pre_f_x = pp.pre_f_x;
	}
	if(min_pre_f_x > pp.pre_f_x)
	{
	    min_pre_f_x = pp.pre_f_x;
	}
	if(max_pre_f_y < pp.pre_f_y)
	{
	    max_pre_f_y = pp.pre_f_y;
	}
	if(min_pre_f_y > pp.pre_f_y)
	{
	    min_pre_f_y = pp.pre_f_y;
	}
	if(max_x < pp.x)
	{
	    max_x = pp.x;
	}
	if(min_x > pp.x)
	{
	    min_x = pp.x;
	}
	if(max_y < pp.y)
	{
	    max_y = pp.y;
	}
	if(min_y > pp.y)
	{
	    min_y = pp.y;
	}
//	if(max_time < pp.time)
//	{
//	    max_time=pp.time;
//	}
//	if(min_time > pp.time)
//	{
//	    min_time=pp.time;
//	}
    }
    
    public void set_size(int size)
    {
        if(fixed_size <= 0)
        {
            if(null != v)
            {
                v = new Vector<PlotPoint>();
            }
            if(v.size() > size)
            {
                v.setSize(size);
            }
            if(current_size > size)
            {
                current_size = size;
            }
        }
    }
    
    public void RecheckAllPoints()
    {
	max_x = Double.NEGATIVE_INFINITY;
	min_x = Double.POSITIVE_INFINITY;
	max_y = Double.NEGATIVE_INFINITY;
	min_y = Double.POSITIVE_INFINITY;
	
	max_orig_x = Double.NEGATIVE_INFINITY;
	min_orig_x = Double.POSITIVE_INFINITY;
	max_orig_y = Double.NEGATIVE_INFINITY;
	min_orig_y = Double.POSITIVE_INFINITY;
	
	max_pre_f_x = Double.NEGATIVE_INFINITY;
	min_pre_f_x = Double.POSITIVE_INFINITY;
	max_pre_f_y = Double.NEGATIVE_INFINITY;
	min_pre_f_y = Double.POSITIVE_INFINITY;
	
	min_time = Double.POSITIVE_INFINITY;
	max_time = Double.NEGATIVE_INFINITY;
	
        if(fixed_size > 0)
        {
            for(int i = 0; i < plot_point_array.length; i++)
            {
                PlotPoint pp = plot_point_array[i];
                CheckPoint(pp);
            }
        }
        else
        {
            for(int i = 0; i < v.size(); i++)
            {
                PlotPoint pp = v.elementAt(i);
                CheckPoint(pp);
            }
        }
    }
    
    public String toString()
    {
	String s= super.toString()+"\n";
	s += "name="+name+"\n";
	s += "short_name="+short_name+"\n";
	s += "current_size="+current_size+"\n";
	s += "v_offset="+v_offset+"\n";
	s += "painting="+painting+"\n";
	s += "p2_count="+p2_count+"\n";
	s += "last_pp_set="+last_pp_set+"\n";
	s += "last_pp_set_index="+last_pp_set_index+"\n";
        if(v != null)
	{
	    s+= "\tv.size()="+v.size()+"\n";
	    if(print_v || v.size() < 4)
	    {
		for(int i = 0; i < v.size(); i++)
		{
		    PlotPoint pp = v.elementAt(i);
		    s += "v.elementAt("+i+") ="+pp+"\n";
		}		
	    }
	    s+="max_orig_x="+max_orig_x+"\n";
	    s+="min_orig_x="+min_orig_x+"\n";
	    s+="max_orig_y="+max_orig_y+"\n";
	    s+="min_orig_y="+min_orig_y+"\n";
	    s+="max_pre_f_x="+max_pre_f_x+"\n";
	    s+="min_pre_f_x="+min_pre_f_x+"\n";
	    s+="max_pre_f_y="+max_pre_f_y+"\n";
	    s+="min_pre_f_y="+min_pre_f_y+"\n";
	    s+="max_x="+max_x+"\n";
	    s+="min_x="+min_x+"\n";
	    s+="max_y="+max_y+"\n";
	    s+="min_y="+min_y+"\n";
	    s+="min_time="+min_time+"\n";
	    s+="max_time="+max_time+"\n";
	}
	else
	{
	    s += "v=null\n";
	}
	s+= "\n}\n";
	return s;
    }
    
    @SuppressWarnings("unchecked")
    public void setPlotPointAt(PlotPoint pp, int i)
    {
	//pp.time=System.currentTimeMillis();
	CheckPoint(pp);
        if(fixed_size > 0)
        {
            if(plot_point_array.length > i && i >= 0)
            {
                plot_point_array[i] = pp;
            }
        }
        else
        {
            if(v.size() <= i)
            {
                last_pp_set_index=v.size();
                v.add(pp);
                current_size = v.size();
            }
            else
            {
                v.setElementAt(pp,i);
                last_pp_set_index=i;
            }
        }
	last_pp_set = pp;
    }
    
    int last_added_plot_point_index = 0;
    
    public void addPlotPoint(PlotPoint pp)
    {
        if(fixed_size > 0)
        {
            last_added_plot_point_index++;
            last_added_plot_point_index %= fixed_size;
            setPlotPointAt(pp,last_added_plot_point_index);
            return;
        }
	//pp.time=System.currentTimeMillis();
	last_pp_set_index=v.size();
	CheckPoint(pp);
	v.addElement(pp);
	last_pp_set = pp;
    }
    
    public int v_out_of_range_count=0;
    
//    public final List<PlotPoint> get_v()
//    {
//        if(fixed_size > 0)
//        {
//            v = new Vector<PlotPoint>().copyInto(plot_point_array)
//            return Collections.unmodifiableList();
//        }
//        else
//        {
//            return Collections.unmodifiableList(v);
//        }
//    }
    
    
    public PlotPoint getPlotPointAt(int i)
    {
        if(fixed_size > 0)
        {
            if(null == plot_point_array[i])
            {
                plot_point_array[i] = new PlotPoint();
            }
            return plot_point_array[i];
        }
	try
	{
	    return v.elementAt(i);
	}
	catch(Exception e)
	{
	    v_out_of_range_count++;
	    if(v_out_of_range_count <2)
	    {
		System.err.println("i="+i);
		System.err.println("this="+this);
		e.printStackTrace();
	    }
	    else
	    {
		System.err.println("PlotData.getPlotPointAt() : v_out_of_range_count="+v_out_of_range_count+", i="+i+", name="+name+", current_size="+current_size);
	    }
	}
	return null;
    }
    
    public int v_size()
    {
        if(fixed_size > 0)
        {
            return fixed_size;
        }
	if(null == v)
	{
	    current_size = 0;
	    return 0;
	}
	if(current_size > v.size())
	{
	    current_size = v.size();
	}
	return v.size();
    }
    
    public void set_v(Vector<PlotPoint> new_v)
    {
	v=new_v;
    }
    
    public void clear_v()
    {
	if(null != v)
	{
	    v.removeAllElements();
	}
	v_offset=0;
	current_size=0;
	p2_count=0;
	last_pp_set=null;
	last_pp_set_index=-1;
    }
    
}
