/*
The NIST RCS (Real-time Control Systems)
library is public domain software, however it is preferred
that the following disclaimers be attached.

Software Copywrite/Warranty Disclaimer

This software was developed at the National Institute of Standards and
Technology by employees of the Federal Government in the course of their
official duties. Pursuant to title 17 Section 105 of the United States
Code this software is not subject to copyright protection and is in the
public domain. NIST Real-Time Control System software is an experimental
system. NIST assumes no responsibility whatsoever for its use by other
parties, and makes no guarantees, expressed or implied, about its
quality, reliability, or any other characteristic. We would appreciate
acknowledgement if the software is used. This software can be
redistributed and/or modified freely provided that any derivative works
bear some notice that they are derived from it, and any modified
versions bear some notice that they have been modified.



 */
package diagapplet.plotter;

import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import java.text.DecimalFormat;
import javax.swing.JPanel;

/**
 * JPanel for the actual graph without the controls around the outside.
 * @author Will Shackleford <shackle@nist.gov>
 */
public class PlotGraphJPanel
		extends JPanel //implements MouseListener, MouseMotionListener
{

	/**
	 *   Unique id for this class, Exception implements Serializable interface see documentation
	 * for serializable interface for recommendations regarding this variable.
	 */
	private static final long serialVersionUID = 2613905L;
	static public boolean use_buffer = false;
	static public int max_points_per_plot = -1; // -1 = no limit
	PlotGraphScreenMap screen_map = null;
	PlotGraphScreenMap array_mode_screen_map = null;
	static public Object SyncObject = new Object();
	public int plotter_num = -1;
	private int max_scroll_x = 0;
	private int max_scroll_y = 0;
	public int repaint_count = 0;
	public int paint_count = 0;
	boolean repaint_just_issued = false;
	String line_style = "11011";
	int line_width = 1;
	int last_plots_to_show = 0;
	static public int repaint_max_tm_millis = 500;
	private boolean array_mode = false;
	public boolean show_lines = true;
	public volatile boolean show_key = true;
	public volatile boolean k2_mode = false;
	public volatile boolean s_mode = false;
	public volatile boolean c_mode = true;
	public volatile boolean l_mode = false;
	public volatile boolean e_mode = false;
	public volatile boolean xy_mode = false;
	public boolean repaint_needed = true;
	public boolean rescale_to_selected_rectangle_needed = false;
	public boolean show_grid = true;
	public boolean show_axis = true;
	public boolean label_grid = true;
	public boolean mark_points = true;
	public boolean label_points = false;
	public static Color axis_color = Color.WHITE;
	public static Color grid_color = Color.BLUE;
	public static Color back_color = Color.BLACK;
	public static Hashtable<String, PlotData> plots = null;
	public static Vector plot_vector = null;
	public static boolean m_color = true;
	public volatile boolean show_rect = false;
	Rectangle selected_rectangle = null;
	static public boolean debug_on = false;
	public static final int ANGLE_DEGREE_UNITS = 1;
	public static final int ANGLE_RAD_UNITS = 2;
	public static final int ANGLE_RADPI_UNITS = 3;
	public int angle_unit_type = ANGLE_DEGREE_UNITS;
	public int radius_lines = 8;
	public int angle_lines = 8;
	public static Color zero_rad_color = null;
	public static DecimalFormat df_noexponent = null;
	public static DecimalFormat df_exponent = null;
	public boolean set_l_mode_on_paint = false;
	private static Color current_line_color;
	private static Color current_point_color;

	public BufferedImage plotToImage() {
		BufferedImage bi = new BufferedImage(this.getWidth(), this.getHeight(), BufferedImage.TYPE_3BYTE_BGR);
		this.paintGraph(bi.getGraphics());
		return bi;
	}

	public BufferedImage plotToImage(final int _width, final int _height) {

		PlotGraphScreenMap cur_screen_map = screen_map;
		if (array_mode) {
			cur_screen_map = array_mode_screen_map;
		}
		final int orig_width = cur_screen_map.get_screen_width();
		final int orig_height = cur_screen_map.get_screen_height();
		cur_screen_map.set_screen_w_x_h(_width, _height);
		if (this.e_mode) {
			cur_screen_map.equalizeAxis();
		}
//		System.out.println("cur_screen_map.get_screen_width() = " + cur_screen_map.get_screen_width());
		System.out.println("cur_screen_map.get_x_scale() = " + cur_screen_map.get_x_scale());
		System.out.println("cur_screen_map.get_y_scale() = " + cur_screen_map.get_y_scale());
		try {
			BufferedImage bi = new BufferedImage(_width, _height, BufferedImage.TYPE_3BYTE_BGR);
			this.paintGraph(bi.getGraphics());
			return bi;
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		} finally {
			cur_screen_map.set_screen_w_x_h(orig_width, orig_height);
		}
	}


	static {
		try {
			df_noexponent = new DecimalFormat("###.###");
			try {
				df_exponent = new DecimalFormat("0.####E0");
			} catch (Exception e) {
				df_exponent = df_noexponent;
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public String toString() {
		String s = super.toString() + "\n";

		s += "plotter_num  =" + plotter_num + "\n";
		s += "max_scroll_x=" + max_scroll_x + "\n";
		s += "max_scroll_y=" + max_scroll_y + "\n";
		s += "repaint_count=" + repaint_count + "\n";
		s += "paint_count=" + paint_count + "\n";
		s += "max_points_per_plot=" + max_points_per_plot + "\n";
		s += "c_mode=" + c_mode + "\n";
		s += "s_mode=" + s_mode + "\n";
		s += "l_mode=" + l_mode + "\n";
		if (screen_map != null) {
			s += "screen_map=" + screen_map + "\n";
		} else {
			s += "screen_map=null;";
		}
		if (array_mode_screen_map != null) {
			s += "array_mode_screen_map=" + array_mode_screen_map + "\n";
		} else {
			s += "array_mode_screen_map=null;";
		}
		return s;
	}

	static public void DebugPrint2(String s) {
		try {
			Throwable t = new Throwable();
			StackTraceElement ste[] = t.getStackTrace();
			System.out.println(ste[1].getFileName() + ":" + ste[1].getLineNumber() + " " + s);
			System.out.println("time=" + System.currentTimeMillis());
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	static public void DebugPrint(String s) {
		try {
			if (!debug_on) {
				return;
			}
			Throwable t = new Throwable();
			StackTraceElement ste[] = t.getStackTrace();
			System.out.println(ste[1].getFileName() + ":" + ste[1].getLineNumber() + " " + s);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	static public void ErrorPrint(String s) {
		try {
			Throwable t = new Throwable();
			StackTraceElement ste[] = t.getStackTrace();
			if (debug_on) {
				System.out.println("ErrorPrint + " + ste[1].getFileName() + ":" + ste[1].getLineNumber() + " " + s);
			}
			System.err.println(ste[1].getFileName() + ":" + ste[1].getLineNumber() + " " + s);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public static String format_double(double d) {
		try {
			if (Math.abs(d) <= 0.001 || Math.abs(d) >= 10000.0) {
				if (null == df_exponent) {
					return Double.toString(d);
				}
				return df_exponent.format(d);
			} else {
				if (null == df_noexponent) {
					return Double.toString(d);
				}
				return df_noexponent.format(d);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return Double.toString(d);
	}

	public void set_array_mode(boolean _array_mode) {
		array_mode = _array_mode;
	}

	public boolean get_array_mode() {
		return array_mode;
	}

	public void Zoom(double scale_factor) {
		if (array_mode) {
			array_mode_screen_map.zoom_x(scale_factor);
			if (!s_mode) {
				array_mode_screen_map.zoom_y(scale_factor);
			}
			if (e_mode) {
				array_mode_screen_map.equalizeAxis();
			}
		} else {
			screen_map.zoom_x(scale_factor);
			if (!s_mode) {
				screen_map.zoom_y(scale_factor);
			}
			if (e_mode) {
				screen_map.equalizeAxis();
			}
		}
	}

	public void ZoomOut() {
		Zoom(2.0);
	}

	public void ZoomIn() {
		Zoom(0.5);
	}

	public void ScrollRight() {
		double min_orig_x = Double.POSITIVE_INFINITY;
		double max_orig_x = Double.NEGATIVE_INFINITY;
		int nump = 0;

		if (array_mode) {
			return;
		}
		for (Object obj : plots.values()) {
			PlotData pd = (PlotData) obj;
			if (pd.array_type != array_mode) {
				continue;
			}
			if (min_orig_x > pd.min_orig_x) {
				min_orig_x = pd.min_orig_x;
			}
			if (max_orig_x < pd.max_orig_x) {
				max_orig_x = pd.max_orig_x;
			}
			nump++;
		}
		if (nump > 0) {
			screen_map.set_abs_x_min(min_orig_x);
			screen_map.set_abs_x_max(max_orig_x);
			double x_diff = screen_map.get_x_max() - screen_map.get_x_min();
			screen_map.set_x_show_area(max_orig_x - x_diff, max_orig_x);
		//screen_map.set_x_show_area(min_orig_x,max_orig_x);
		}
		if (e_mode) {
			screen_map.equalizeAxis();
		}
	}

	public void FitToGraph() {
		double min_orig_x = Double.POSITIVE_INFINITY;
		double max_orig_x = Double.NEGATIVE_INFINITY;
		double min_orig_y = Double.POSITIVE_INFINITY;
		double max_orig_y = Double.NEGATIVE_INFINITY;
		int nump = 0;

		PlotData min_x_pd = null;
		PlotData max_x_pd = null;
		PlotData min_y_pd = null;
		PlotData max_y_pd = null;

		for (Object obj : plots.values()) {
			PlotData pd = (PlotData) obj;
			if (pd.array_type != array_mode) {
				continue;
			}
			if (null != pd.show &&
					plotter_num >= 0 && plotter_num < pd.show.length &&
					!pd.show[plotter_num]) {
				continue;
			}
			pd.RecheckAllPoints();
			if (min_orig_x > pd.min_orig_x) {
				min_x_pd = pd;
				min_orig_x = pd.min_orig_x;
			}
			if (max_orig_x < pd.max_orig_x) {
				max_orig_x = pd.max_orig_x;
				max_x_pd = pd;
			}
			if (min_orig_y > pd.min_orig_y) {
				min_orig_y = pd.min_orig_y;
				min_y_pd = pd;
			}
			if (max_orig_y < pd.max_orig_y) {
				max_orig_y = pd.max_orig_y;
				max_y_pd = pd;
			}
			nump++;
		}
//	if(nump > 1)
//	{
//	    System.out.println("min_orig_x="+min_orig_x);
//	    System.out.println("min_x_pd="+min_x_pd);
//	    System.out.println("max_orig_x="+max_orig_x);
//	    System.out.println("max_x_pd="+max_x_pd);
//	    System.out.println("min_orig_y="+min_orig_y);
//	    System.out.println("min_y_pd="+min_y_pd);
//	    System.out.println("min_orig_y="+max_orig_y);
//	    System.out.println("max_y_pd="+max_y_pd);	    
//	}
		if (nump > 0) {
			if (array_mode) {
				array_mode_screen_map.set_abs_x_min(min_orig_x);
				array_mode_screen_map.set_abs_x_max(max_orig_x);
				array_mode_screen_map.set_x_show_area(min_orig_x, max_orig_x);
				if (!s_mode) {
					array_mode_screen_map.set_abs_y_min(min_orig_y);
					array_mode_screen_map.set_abs_y_max(max_orig_y);
					array_mode_screen_map.set_y_show_area(min_orig_y, max_orig_y);
				}
				if (e_mode) {
					array_mode_screen_map.equalizeAxis();
				}
			} else {
				screen_map.set_abs_x_min(min_orig_x);
				screen_map.set_abs_x_max(max_orig_x);
				screen_map.set_x_show_area(min_orig_x, max_orig_x);
				if (!s_mode) {
					screen_map.set_abs_y_min(min_orig_y);
					screen_map.set_abs_y_max(max_orig_y);
					screen_map.set_y_show_area(min_orig_y, max_orig_y);
				}
				if (e_mode) {
					screen_map.equalizeAxis();
				}
			}
		}
	}

	public void FitY() {
		double min_orig_y = Double.POSITIVE_INFINITY;
		double max_orig_y = Double.NEGATIVE_INFINITY;
		int nump = 0;
		if (s_mode || array_mode) {
			return;
		}

		for (Object obj : plots.values()) {
			PlotData pd = (PlotData) obj;
			if (pd.array_type != array_mode) {
				continue;
			}
			if (null != pd.show &&
					!pd.show[plotter_num]) {
				continue;
			}
			pd.RecheckAllPoints();
			if (min_orig_y > pd.min_orig_y) {
				min_orig_y = pd.min_orig_y;
			}
			if (max_orig_y < pd.max_orig_y) {
				max_orig_y = pd.max_orig_y;
			}
			nump++;
		}
		if (nump > 0) {
			if (!s_mode) {
				screen_map.set_abs_y_min(min_orig_y);
				screen_map.set_abs_y_max(max_orig_y);
				screen_map.set_y_show_area(min_orig_y, max_orig_y);
				if (e_mode) {
					screen_map.equalizeAxis();
				}
			}
		}
	}

	public PlotGraphJPanel() {
		Dimension d = getSize();
		if (d.width > 1 && d.height > 1) {
			screen_map = new PlotGraphScreenMap(d.width, d.height);
			array_mode_screen_map = new PlotGraphScreenMap(d.width, d.height);
		} else {
			screen_map = new PlotGraphScreenMap(600, 300);
			array_mode_screen_map = new PlotGraphScreenMap(600, 300);
		}

		//      System.out.println("start_width="+start_width+",start_height="+start_height+", getSize() = "+getSize());
		//Thread.dumpStack();
		try {
			Font f = getFont();
			if (f != null) {
				if (f.getSize() < 12) {
					setFont(f.deriveFont(12));
				}
			} else {
				setFont(new Font("SanSerif", Font.PLAIN, 20));
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		repaint_needed = true;
		image_needs_to_be_updated = true;
		repaint_just_issued = false;
		selected_rectangle = new Rectangle();
		show_rect = false;
		//	changeScale(x_min, x_max, scroll_width, y_min, y_max, scroll_height);
		if (null == plots) {
			plots = new Hashtable<String, PlotData>();
		}
		if (use_buffer) {
			bufferImage = createImage(d.width, d.height);
		}
	}
	int plot_clicking_on = -1;
	long time_of_last_click = 0;

	public void SetReverseX(boolean _new_reverse_x) {
		if (null != this.screen_map) {
			this.screen_map.SetReverseX(_new_reverse_x);
		}
	}

	public void mouseClicked(MouseEvent evt) {
		int x = evt.getX();
		int y = evt.getY();
		if (show_key && !s_mode) {
			if (x > 30 && x < 70 && y > 10 && y < (keyVector.size() + 1) * 25 + 25) {
				int new_plot_clicking_on = (y - 20) / 25;
				//System.out.println("new_plot_clicking_on="+new_plot_clicking_on+", keyVector.size()="+keyVector.size());
				if (new_plot_clicking_on < 0) {
					new_plot_clicking_on = 0;
				}
				if (new_plot_clicking_on > keyVector.size() - 1) {
					new_plot_clicking_on = keyVector.size() - 1;
				}
				PlotData pd = keyVector.elementAt(new_plot_clicking_on);
				if (null == pd) {
					return;
				}
				if (null == pd.show) {
					pd.show = new boolean[plotter_num + 1];
				}
				if (pd.show.length <= plotter_num) {
					//   System.out.println("pd.show.length="+pd.show.length+", plotter_num  ="+plotter_num  );
					boolean new_show[] = new boolean[plotter_num + 1];
					for (int i = 0; i < pd.show.length; i++) {
						new_show[i] = pd.show[i];
					}
					for (int i = pd.show.length; i < plotter_num + 1; i++) {
						new_show[i] = true;
					}
					pd.show = new_show;
				}
				pd.show[plotter_num] = !pd.show[plotter_num];
			//System.out.println("pd.show[plotter_num  ="+plotter_num  +"] = "+pd.show[plotter_num  ]);
			}
		}
		plot_clicking_on = -1;
	}

	public void mouseEntered(MouseEvent evt) {
	}

	public void mouseExited(MouseEvent evt) {
	}

	public void mousePressed(MouseEvent evt) {
		int x = evt.getX();
		int y = evt.getY();
		last_x = x;
		last_y = y;
		repaint_needed = true;
		selected_rectangle.x = x;
		selected_rectangle.y = y;
		selected_rectangle.width = 0;
		selected_rectangle.height = 0;
		show_rect = true;
	}

	public void mouseMoved(MouseEvent evt) {
		last_x = evt.getX();
		last_y = evt.getY();
	}

	public void mouseDragged(MouseEvent evt) {
		int x = evt.getX();
		int y = evt.getY();
		last_x = x;
		last_y = y;
		int temp;
		if (debug_on) {
			System.out.println("mouseDragged(" + evt + ") x=" + x + ", y=" + y);
		}
		repaint_needed = true;
		if (x > selected_rectangle.x) {
			selected_rectangle.width = x - selected_rectangle.x;
		} else {
			temp = selected_rectangle.x;
			selected_rectangle.x = x;
			selected_rectangle.width += temp - selected_rectangle.x;
		}
		if (y > selected_rectangle.y) {
			selected_rectangle.height = y - selected_rectangle.y;
		} else {
			temp = selected_rectangle.y;
			selected_rectangle.y = y;
			selected_rectangle.height += temp - selected_rectangle.y;
		}
		show_rect = true;
		repaint();
	}

	public void mouseReleased(MouseEvent evt) {
		int x = evt.getX();
		int y = evt.getY();
		last_x = x;
		last_y = y;
		if (debug_on) {
			System.out.println("mouseReleased(" + evt + ") x=" + x + ", y=" + y);
		}
		if (show_rect && selected_rectangle.width > 5 && selected_rectangle.height > 5) {
			rescale_to_selected_rectangle_needed = true;
			show_rect = false;
			repaint_just_issued = false;
			//System.out.println("mouseUp setting repaint_needed = true");
			repaint_needed = true;
			return;
		}
		show_rect = false;
		image_needs_to_be_updated = true;
		repaint();
	}

	static public boolean ColorTooClosePreviousPlot(Color c1, long min_diff) {
		try {
			if (null == plots) {
				return false;
			}
			//long min_diff = 400 / (plots.size() + 1);
			for (PlotData pd : plots.values()) {
				if (ColorsTooClose(c1, pd.line_color, min_diff) ||
						ColorsTooClose(c1, pd.point_color, min_diff)) {
					return true;
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return false;
	}

	public static boolean ColorsTooClose(Color c1, Color c2, long min_diff) {
		try {
			if (null == c2) {
				return false;
			}
			long red_diff = Math.abs(c1.getRed() - c2.getRed());
			long green_diff = Math.abs(c1.getGreen() - c2.getGreen());
			long blue_diff = Math.abs((c1.getBlue() - c2.getBlue()));
			long total_diff = red_diff + blue_diff + green_diff;
//			if (min_diff > 25) {
//				System.out.println("c1 = " + c1);
//				System.out.println("c2 = " + c2);
//				System.out.println("min_diff = " + min_diff);
//				if (true) {
//					System.out.print("red_diff=" + red_diff + ", green_diff=" + green_diff + ", blue_diff=" + blue_diff + ", total_diff=" + total_diff + "\n");
//				}
//			}
			if (total_diff < min_diff) {
				return true;
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return false;
	}

	public static void AddPlot(PlotData pd, String name) {
		if (null == name || clearing_plots) {
			return;
		}
		if (name.length() < 1) {
			return;
		}
		pd.name = name;
		AddPlot(pd);
	}

	@SuppressWarnings("unchecked")
	public static void AddPlot(PlotData pd) {
		if (debug_on) {
			DebugPrint("Adding  plot " + pd + ", " + pd.name);
			DebugPrint(" to " + plots);
		//Thread.dumpStack();
		}
		final String name = pd.name;
		if (null == pd) {
			return;
		}
		if (pd.name.length() < 30) {
			pd.short_name = pd.name;
		} else {
			int p_index = name.lastIndexOf('.');
			int short_name_start = p_index;
			int pcount = 0;
			while (p_index > 0) {
				int next_length = name.length() - p_index;
				if (next_length > 50 || next_length > 30 && pcount > 3) {
					break;
				}
				short_name_start = p_index;
				p_index = name.lastIndexOf('.', p_index - 1);
				pcount++;
			}
			if (pcount < 3 && name.length() < 50) {
				short_name_start = 0;
			}
			if (short_name_start > 0 && short_name_start < name.length() - 1) {
				pd.short_name = name.substring(short_name_start + 1);
			} else {
				pd.short_name = name;
			}
		}

		current_line_color = PlotterCommon.nextColor();
		Color orig_color_line_color = current_line_color;
		int tries = 0;
		while (ColorsTooClose(current_line_color, back_color, 500 - tries * 5) ||
				ColorsTooClose(current_line_color, grid_color, 50 - tries) ||
				ColorsTooClose(current_line_color, axis_color, 50 - tries) ||
				ColorTooClosePreviousPlot(current_line_color, (800 / (plots.size() + 1 + (tries / 5))))) {
			current_line_color = PlotterCommon.nextColor();
			tries++;
			if (current_line_color == orig_color_line_color || tries > 40) {
				break;
			}
		}
		current_point_color = current_line_color;
		if (null == plots) {
			plots = new Hashtable();
		}
		if (!plots.containsKey(name)) {
			if (debug_on) {
				DebugPrint("Adding  plot " + name);
			}
			if (null == pd.line_color) {
				pd.line_color = current_line_color;
			}
			if (null == pd.point_color) {
				pd.point_color = current_point_color;
			}
			if (pd.line_width < 1) {
				pd.line_width = (plots.size() % 4) + 1;
			}
		} else {
			PlotData old_value = plots.get(name);
			if (null == pd.line_color) {
				pd.line_color = old_value.line_color;
			}
			if (null == pd.point_color) {
				pd.point_color = old_value.point_color;
			}
			if (pd.line_width < 1) {
				pd.line_width = old_value.line_width;
			}
			if (null == pd.line_style) {
				pd.line_style = old_value.line_style;
			}
		}
		AddPlotStep2(pd);
		if (debug_on) {
			DebugPrint("Added plot " + pd + ", " + name);
		}
	}

	private static void AddPlotStep2(PlotData pd) {
		if (debug_on) {
			DebugPrint("pd =" + pd + ", name=" + pd.name + ", plots=" + plots);
		}
		// for(int i = 0; i < dplotter.dplotter_vector.size(); i++)
// 	{
// 	    dplotter dp = (dplotter) dplotter.dplotter_vector.elementAt(i);
// 	    if(null != dp)
// 	    {
// 		if(null != dp.plot_graph)
// 		{
// 		    dp.plot_graph.repaint_needed = true;
// 		    dp.plot_graph.repaint_just_issued = false;
// 		}
// 	    }
// 	}



		plots.put(pd.name, pd);
		if (null == keyVector) {
			keyVector = new Vector<PlotData>();
		}
		keyVector.removeAllElements();
		Enumeration plots_enum = plots.elements();
		while (plots_enum.hasMoreElements()) {
			PlotData pd2 = (PlotData) plots_enum.nextElement();
			//DebugPrint2("pd2="+pd2+", pd2.name="+pd2.name);
			keyVector.addElement(pd2);
		}
		for (int i = 0; i < keyVector.size(); i++) {
			for (int j = i + 1; j < keyVector.size(); j++) {
				PlotData pdi = keyVector.elementAt(i);
				PlotData pdj = keyVector.elementAt(j);
				if (pdi.name.compareTo(pdj.name) > 0) {
					keyVector.setElementAt(pdi, j);
					keyVector.setElementAt(pdj, i);
				}

			}
		}
	}

	public PlotData GetPlot(String name) {
		try {
			if (null != plots) {
				return plots.get(name);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	public static void RemoveAllPlots() {
		plots = new Hashtable<String, PlotData>();
		if (null != keyVector) {
			keyVector.removeAllElements();
		}
		keyVector = new Vector<PlotData>();
	}
	static boolean clearing_plots = false;

	public static void ClearAllData() {
		try {
			clearing_plots = true;
			if (null != keyVector) {
				for (int ki = 0; ki < keyVector.size(); ki++) {
					PlotData plot_data = keyVector.elementAt(ki);
					plot_data.clear_v();
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		clearing_plots = false;
	}

	public void RemovePlot(String name) {
		plots.remove(name);
		if (null != keyVector) {
			keyVector.removeAllElements();
			Enumeration plots_enum = plots.elements();
			while (plots_enum.hasMoreElements()) {
				PlotData pd2 = (PlotData) plots_enum.nextElement();
				keyVector.addElement(pd2);
			}
		}
		image_needs_to_be_updated = true;
	}

	public static void AddPointToPlot(PlotData pd, double x, double y, boolean connected, double pre_f_x, double pre_f_y) {
		if (clearing_plots) {
			return;
		}
		if (pd.delete_me) {
			return;
		}
		// for(int i = 0; i < dplotter.dplotter_vector.size(); i++)
// 	{
// 	    dplotter dp = (dplotter) dplotter.dplotter_vector.elementAt(i);
// 	    if(null != dp)
// 	    {
// 		if(null != dp.plot_graph)
// 		{
// 		    dp.plot_graph.repaint_needed = true;
// 		    dp.plot_graph.repaint_just_issued = false;
// 		}
// 	    }
// 	}
		if (max_points_per_plot <= 0 || max_points_per_plot > pd.v_size()) {
			pd.addPlotPoint(new PlotPoint((int) x, (int) y, x, y, pre_f_x, pre_f_y));
			pd.current_size++;
		} else {
			int v_index = (pd.v_offset + max_points_per_plot) % pd.v_size();
			PlotPoint pp = pd.getPlotPointAt(v_index);
			pp.x = (int) x;
			pp.y = (int) y;
			pp.orig_x = x;
			pp.orig_y = y;
			pp.pre_f_x = pre_f_x;
			pp.pre_f_y = pre_f_y;
			//pp.connected = connected;
			pd.setPlotPointAt(pp, v_index);
			if (pd.current_size >= pd.v_size()) {
				pd.current_size = pd.v_size();
			}
			if (pd.current_size >= max_points_per_plot) {
				pd.v_offset = (pd.v_offset + 1) % pd.v_size();
			} else {
				pd.current_size++;
			}
		}
	}

	public static void AddPointToArrayPlot(PlotData pd, int index, double x, double y, double pre_f_x, double pre_f_y) {
		if (clearing_plots) {
			return;
		}
		if (pd.delete_me) {
			return;
		}
		// for(int i = 0; i < dplotter.dplotter_vector.size(); i++)
// 	{
// 	    dplotter dp = (dplotter) dplotter.dplotter_vector.elementAt(i);
// 	    if(null != dp)
// 	    {
// 		if(null != dp.plot_graph)
// 		{
// 		    dp.plot_graph.repaint_needed = true;
// 		    dp.plot_graph.repaint_just_issued = false;
// 		}
// 	    }
// 	}
		PlotPoint pp = null;
		if (pd.v_size() > index) {
			pp = pd.getPlotPointAt(index);
		}
		if (null == pp) {
			pp = new PlotPoint((int) x, (int) y, x, y, pre_f_x, pre_f_y);
		} else {
			pp.x = (int) x;
			pp.y = (int) y;
			pp.orig_x = x;
			pp.orig_y = y;
			pp.pre_f_x = pre_f_x;
			pp.pre_f_y = pre_f_y;
		// pp.connected = true;
		}
		pd.setPlotPointAt(pp, index);
	}

	public void equalizeAxis() {
		image_needs_to_be_updated = true;
		try {
			if (array_mode) {
				array_mode_screen_map.equalizeAxis();
			} else {
				screen_map.equalizeAxis();
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void UpdateDisplay(boolean force_repaint) {
		if (force_repaint) {
			image_needs_to_be_updated = true;
			repaint_needed = true;
		}

		if (array_mode) {
			if (array_mode_screen_map.get_changed()) {
				image_needs_to_be_updated = true;
				repaint_needed = true;
				array_mode_screen_map.clear_changed();
			}
		} else {
			if (screen_map.get_changed()) {
				image_needs_to_be_updated = true;
				repaint_needed = true;
				screen_map.clear_changed();
			}
		}
		if (show_rect && use_buffer) {
			repaint_just_issued = false;
			repaint_count++;
			repaint(repaint_max_tm_millis);
			return;
		}
		if (repaint_needed && !repaint_just_issued) {
			if (!show_rect) {
				image_needs_to_be_updated = true;
				repaint_just_issued = true;
			}
			repaint_count++;
			repaint(repaint_max_tm_millis);
		}
	}
	public static Vector<PlotData> keyVector = null;
	int last_x;
	int last_y;
	public static String extra_sh_str = null;
	public static String short_extra_sh_str = null;

	private void paintKey(Graphics g) {
		try {
			if (null == keyVector) {
				return;
			}
			if (m_color) {
				g.setColor(Color.white);
			} else {
				g.setColor(Color.black);
			}
			int y_pos = 20;
			int x_pos = 30;
			if (plots != null) {
				for (int i = 0; i < keyVector.size(); i++) {
					String shStr = " (hidden)";
					PlotData pd = keyVector.elementAt(i);
					if (pd.no_key) {
						continue;
					}
					boolean show_this_plot = false;
					if (pd.show == null) {
						shStr = " (visible)";
						show_this_plot = true;
					} else {
						if (pd.show == null || pd.show.length <= plotter_num || plotter_num < 0) {
							shStr = " (visible)";
							show_this_plot = true;
						} else {
							if (pd.show[plotter_num]) {
								shStr = " (visible)";
								show_this_plot = true;
							}

						}
					}
					if (null != pd.add_to_key) {
						shStr += pd.add_to_key;
					}
					if (null != extra_sh_str) {
						shStr += extra_sh_str;
					}
					if (pd.v_size() < 1 || pd.name.length() < 1) {
						continue;
					}
					if (show_this_plot) {
						if (c_mode) {
							if (m_color) {
								g.setColor(pd.line_color);
							}
							g.drawLine(x_pos, y_pos, x_pos + 40, y_pos);
						}
						if (m_color) {
							g.setColor(pd.point_color);
						}
						g.fillRect(x_pos + 20, y_pos - 2, 4, 4);
					} else if (c_mode) {
						if (m_color && axis_color != null) {
							g.setColor(axis_color);
						}
						g.drawLine(x_pos + 10, y_pos, x_pos + 30, y_pos);
					}
					if (m_color) {
						g.setColor(pd.point_color);
					}
					g.drawString(pd.name + shStr, x_pos + 50, y_pos + 5);
					y_pos += 25;
				}
			}
			PlotGraphScreenMap cur_screen_map = screen_map;
			if (array_mode) {
				cur_screen_map = array_mode_screen_map;
			}
			double last_x_pos = cur_screen_map.get_x_value(last_x);
			double last_y_pos = cur_screen_map.get_y_value(last_y);
			if (debug_on) {
				DebugPrint("last_x=" + last_x + ", last_x_pos=" + last_x_pos);
				DebugPrint("last_y=" + last_y + ", last_y_pos=" + last_y_pos);
			}
			String pos_string = null;

			// spacing("+cur_screen_map.get_x_grid_spacing()+","+cur_screen_map.get_y_grid_spacing()+")",
			if (k2_mode) {
				if (m_color && axis_color != null) {
					g.setColor(axis_color);
				}
				if (show_rect) {
					pos_string = "{" + cur_screen_map.get_x_value(selected_rectangle.x) + ", " + cur_screen_map.get_y_value(selected_rectangle.y) + "}  (" + last_x_pos + ", " + last_y_pos + ")";
				} else {
					pos_string = "(" + last_x_pos + ", " + last_y_pos + ")";
				}
				g.drawString(pos_string, x_pos + 50, y_pos + 5);
				y_pos += 25;
				g.drawString("grid (" + cur_screen_map.get_x_grid() + "," + cur_screen_map.get_y_grid() + ") : scale(" + cur_screen_map.get_x_scale() + ", " + cur_screen_map.get_y_scale() + ")",
						x_pos + 50, y_pos + 5);

				y_pos += 25;
				g.drawString("abs_x (" + cur_screen_map.get_abs_x_min() + "," + cur_screen_map.get_abs_x_max() + ") : abs_y(" + cur_screen_map.get_abs_y_min() + ", " + cur_screen_map.get_abs_y_max() + ")", x_pos + 50, y_pos + 5);
				y_pos += 25;
				g.drawString("x (" + cur_screen_map.get_x_min() + "," + cur_screen_map.get_x_max() + ") : y(" + cur_screen_map.get_y_min() + ", " + cur_screen_map.get_y_max() + ")", x_pos + 50, y_pos + 5);
				y_pos += 25;
				g.drawString("scroll_x=" + cur_screen_map.get_scroll_x() + ", scroll_width=" + cur_screen_map.get_scroll_width() + ", scroll_y=" + cur_screen_map.get_scroll_y() + ", scroll_height= " + cur_screen_map.get_scroll_height() + ")", x_pos + 50, y_pos + 5);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	int last_x1;
	int last_x2;
	int last_y1;
	int last_y2;
	static final boolean UseOrigDrawLine = true;//    private void EnhancedDrawLine(Graphics  g,int x1, int y1, int x2, int y2)
//    {
//	int temp;
//	int x_low;
//	int x_high;
//	int x_pos;
//	int y_low;
//	int y_high;
//	int y_pos;
//	int x_diff;
//	int y_diff;
//	int i;
//	double m;
//	double inverse_m;
//	int starting_i = 0;
//	try
//	{
//	    if(x1 == last_x1 && x2 == last_x2 && y1 == last_y1 && y2 == last_y2)
//	    {
//		return;
//	    }
//	    if(UseOrigDrawLine)
//	    {
//		g.drawLine(x1,y1,x2,y2);
//		return;
//	    }
//	    PlotGraphScreenMap cur_screen_map = screen_map;
//	    if(array_mode)
//	    {
//		cur_screen_map = array_mode_screen_map;
//	    }
//	    last_x1 = x1;
//	    last_x2 = x2;
//	    last_y1 = y1;
//	    last_y2 = y2;
//	    if(x2 > x1)
//	    {
//		x_low = x1;
//		x_high = x2;
//	    }
//	    else
//	    {
//		x_low = x2;
//		x_high = x1;
//	    }
//	    if(y2 > y1)
//	    {
//		y_low = y1;
//		y_high = y2;
//	    }
//	    else
//	    {
//		y_low = y2;
//		y_high = y1;
//	    }
//	    x_diff = x_high - x_low;
//	    y_diff = y_high - y_low;
//	    
//	    if(x_diff < 3 && y_diff < 3)
//	    {
//		return;
//	    }
//	    // Handle vertical line and sindgle point case separately.
//	    if(x_diff < 1)
//	    {
//		// Handle sindgle point case separately.
//		if(y_diff < 1)
//		{
//		    g.fillRect(x_low,y_low,1,1);
//		    return;
//		}
//		g.fillRect(x_low,y_low,line_width,y_diff);
//		return;
//	    }
//	    // Handle horizontal line case separately.
//	    if(y_diff < 1)
//	    {
//		g.fillRect(x_low,y_low,x_diff,line_width);
//		return;
//	    }
//	    
//	    if(y_diff > x_diff)
//	    {
//		// Swap the points to make the higher point (x1,y1)
//		if(y1 > y2)
//		{
//		    temp = x2;
//		    x2 = x1;
//		    x1 = temp;
//		    temp = y2;
//		    y2 = y1;
//		    y1 = temp;
//		}
//		if(y1 > 0)
//		{
//		    starting_i = 0;
//		}
//		else
//		{
//		    starting_i = -y1;
//		}
//		for(i = starting_i; i < y_diff; i++)
//		{
//		    if(line_style != null)
//		    {
//			if(line_style.charAt(i%line_style.length()) == '0')
//			{
//			    continue;
//			}
//		    }
//		    inverse_m = ((double)(x2 - x1))/((double)(y2 -y1));
//		    y_pos = y1 + i;
//		    if(y_pos > cur_screen_map.get_screen_height())
//		    {
//			break;
//		    }
//		    x_pos = (int) (x1 + inverse_m*i);
//		    if(x_pos > cur_screen_map.get_screen_width()+line_width/2 && inverse_m > 0)
//		    {
//			break;
//		    }
//		    if(x_pos < -line_width/2 && inverse_m < 0)
//		    {
//			break;
//		    }
//		    g.fillRect(x_pos-line_width/2,y_pos,line_width,1);
//		}
//	    }
//	    else
//	    {
//		// Swap the points to make the one to the left (x1,y1)
//		if(x1 > x2)
//		{
//		    temp = x2;
//		    x2 = x1;
//		    x1 = temp;
//		    temp = y2;
//		    y2 = y1;
//		    y1 = temp;
//		}
//		if(x1 > 0)
//		{
//		    starting_i = 0;
//		}
//		else
//		{
//		    starting_i = -x1;
//		}
//		for(i = starting_i; i < x_diff; i++)
//		{
//		    if(line_style != null)
//		    {
//			if(line_style.charAt(i%line_style.length()) == '0')
//			{
//			    continue;
//			}
//		    }
//		    m = ((double)(y2 -y1))/ ((double)(x2 - x1));
//		    x_pos = x1 + i;
//		    if(x_pos > cur_screen_map.get_screen_width())
//		    {
//			break;
//		    }
//		    y_pos = (int) (y1 + m*i);
//		    if(y_pos > cur_screen_map.get_screen_height()+line_width/2 && m > 0)
//		    {
//			break;
//		    }
//		    if(x_pos < -line_width/2 && m < 0)
//		    {
//			break;
//		    }
//		    g.fillRect(x_pos,y_pos-line_width/2,1,line_width);
//		}
//	    }
//	}
//	catch(Exception e)
//	{
//	    e.printStackTrace();
//	}
//    }
	Font font = null;
	FontMetrics fm = null;
	String label_string = null;
	double theta_min_last = 0.0;
	double theta_max_last = 0.0;
	double r_min_last = 0.0;
	double r_max_last = 0.0;
	double r_spacing = 1.0;
	int scale_change = 2;
	private boolean first_paint_cart_grid = true;
	private boolean first_bad_dims = true;
	private boolean first_bad_num_y_lines = true;
	private boolean first_bad_num_x_lines = true;

	public void paintCartesianGrid(Graphics g) {
		try {
			if (m_color && grid_color != null) {
				g.setColor(grid_color);
			}
			PlotGraphScreenMap cur_screen_map = screen_map;
			if (array_mode) {
				cur_screen_map = array_mode_screen_map;
			}
			final int x_grid_spacing = cur_screen_map.get_x_grid_spacing();
			double x_value = cur_screen_map.get_starting_x_grid_value();
			double x_grid = cur_screen_map.get_x_grid();
			final int width = cur_screen_map.get_screen_width();
			final int height = cur_screen_map.get_screen_height();

			final int y_grid_spacing = cur_screen_map.get_y_grid_spacing();
			double y_value = cur_screen_map.get_starting_y_grid_value();
			double y_grid = cur_screen_map.get_y_grid();
			boolean reset_grid_color = false;
			if (width < 10 || width > 200000) {
				if (first_bad_dims) {
					Thread.dumpStack();
					System.err.println("Can't paint cartesian grid with width=" + width + ", and x_grid_spacing=" + x_grid_spacing);
					first_bad_dims = false;
				}
				return;
			}
			if (height < 10 || height > 200000) {
				if (first_bad_dims) {
					System.err.println("Can't paint cartesian grid with height=" + height + ", and y_grid_spacing=" + y_grid_spacing);
					first_bad_dims = false;
				}
				return;
			}
			final int x_grid_start = cur_screen_map.get_starting_x_grid_screen_pos();
			int x_lines_drawn = 0;
			for (int x_pos = x_grid_start; x_pos < 2*width && x_pos > -2*width; x_pos += x_grid_spacing, x_value += x_grid) {
				x_lines_drawn++;
				if (x_lines_drawn > 200) {
					if (first_bad_num_x_lines) {
						System.err.println("Bad num x lines : Can't paint cartesian grid with width=" + width + ", and x_grid_spacing=" + x_grid_spacing);
						first_bad_num_x_lines = false;
					}
					return;
				}
				if (m_color && axis_color != null && show_axis && !s_mode && Math.abs(x_value / x_grid) < 1e-6) {
					g.setColor(axis_color);
					reset_grid_color = true;
				}
				g.drawLine(x_pos, 0, x_pos, height);
				if (reset_grid_color && m_color && grid_color != null) {
					g.setColor(grid_color);
				}
				reset_grid_color = false;
				if (label_grid && Math.abs(x_grid_spacing) > 20) {
					g.drawString(format_double(x_value), x_pos + 5, height - 5);
				}
			}
			if (!s_mode) {
				final int y_start_grid = cur_screen_map.get_starting_y_grid_screen_pos();
				int y_lines_drawn = 0;
				for (int y_pos = y_start_grid;
						y_pos >= -2*height && y_pos < height * 2; y_pos -= y_grid_spacing, y_value += y_grid) {
					y_lines_drawn++;
					if (y_lines_drawn > 200) {
						if (first_bad_num_y_lines) {
							System.err.println("Too many y_lines -- Can't paint cartesian grid with height=" + height + ", y_grid_spacing=" + y_grid_spacing+", and y_start_grid="+y_start_grid);
							first_bad_num_y_lines = false;
						}
						return;
					}
					if (m_color && axis_color != null && show_axis && !s_mode && Math.abs(y_value / y_grid) < 1e-6) {
						g.setColor(axis_color);
						reset_grid_color = true;
					}
					g.drawLine(0, y_pos, width, y_pos);
					if (reset_grid_color && m_color && grid_color != null) {
						g.setColor(grid_color);
					}
					reset_grid_color = false;
					if (label_grid && y_grid_spacing > 20) {
						g.drawString(format_double(y_value), 5, y_pos - 5);
					}
				}
				if (y_lines_drawn < 2) {
					if (first_bad_num_y_lines) {
						System.err.println("Too few y_lines -- Can't paint cartesian grid with height=" + height + ", y_grid_spacing=" + y_grid_spacing+" and y_start_grid="+y_start_grid);
						first_bad_num_y_lines = false;
					}
					return;
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		first_paint_cart_grid = false;
	}
	Image bufferImage = null;
	boolean image_needs_to_be_updated = true;

	protected void paintComponent(Graphics g) {
		if (debug_on) {
			System.out.println("paint() called. paint_count = " + paint_count);
		}
		PlotGraphScreenMap cur_screen_map = screen_map;
		if (array_mode) {
			cur_screen_map = array_mode_screen_map;
		}
		super.paintComponent(g);
		if (clearing_plots) {
			return;
		}
		if (null != back_color) {
			setBackground(back_color);
		}

		if (use_buffer) {
			if (null == bufferImage) {
				image_needs_to_be_updated = true;
			}
			if (image_needs_to_be_updated) {
				if (null == bufferImage) {
					// System.out.println("creating new buffer Image");
					bufferImage = createImage(cur_screen_map.get_screen_width(), cur_screen_map.get_screen_height());
				}
				Graphics image_g = bufferImage.getGraphics();
				if (null == image_g) {
					// System.out.println("bufferImage.getGraphics = null");
					paintGraph(g);
					paintDraggingRect(g);
					return;
				}
				// System.out.println("Painting image.");
				image_g.clearRect(0, 0, cur_screen_map.get_screen_width(), cur_screen_map.get_screen_height());
				paintGraph(image_g);
				image_needs_to_be_updated = false;
			}
			if (null != bufferImage) {
				g.drawImage(bufferImage, 0, 0, cur_screen_map.get_screen_width(), cur_screen_map.get_screen_height(), null);
			}
		} else {
			paintGraph(g);
		}
		paintDraggingRect(g);
	}
	static private int point_size_limit = 20000;

	public static void set_point_size_limit(int _new_point_size_limit) {
		point_size_limit = _new_point_size_limit;
	}

	public static int get_point_size_limit() {
		return point_size_limit;
	}

	public void paintDraggingRect(Graphics g) {
		try {
			// System.out.println("paintDraggingRect() called. show_rect="+show_rect);
			if (show_rect) {
				if (m_color && axis_color != null) {
					g.setColor(axis_color);
				}
				g.drawRect(selected_rectangle.x, selected_rectangle.y, selected_rectangle.width, selected_rectangle.height);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}    //long last_pg_time=0;
	private int last_height = -1;

	public void paintGraph(Graphics g) {

		try {
			if (clearing_plots) {
				return;
			}

			if (null == keyVector) {
				return;
			}
			PlotGraphScreenMap cur_screen_map = screen_map;
			if (array_mode) {
				cur_screen_map = array_mode_screen_map;
			}
			repaint_just_issued = false;
			repaint_needed = false;
			paint_count++;
			if (debug_on) {
				DebugPrint("Repainting PlotGraph plotter_num   = " + plotter_num);
			}
			synchronized (SyncObject) {
				if (m_color) {
					g.setColor(Color.white);
				} else {
					g.setColor(Color.black);
				}
				if (show_grid) {
					paintCartesianGrid(g);
				}
				if (show_key && !s_mode) {
					paintKey(g);
				}

				if (plots != null) {
					if (debug_on) {
						DebugPrint("Repainting PlotGraph plots = " + plots);
					}
					int plots_to_show = 0;
					int width = cur_screen_map.get_screen_width();
					int height = cur_screen_map.get_screen_height();
					if (s_mode) {
						for (int ki = 0; ki < keyVector.size(); ki++) {
							PlotData plot_data = keyVector.elementAt(ki);

							if (null != plot_data.show && plotter_num >= 0) {
								if (plot_data.show.length > plotter_num) {
									if (!plot_data.show[plotter_num] || plot_data.array_type != array_mode) {
										continue;
									}
								}
							}
							plots_to_show++;
						}
						if (plots_to_show < max_plots_to_show || max_plots_to_show < 1) {
							cur_screen_map.set_num_y_sections(plots_to_show);
						} else {
							cur_screen_map.set_num_y_sections(max_plots_to_show);
						}
					} else {
						cur_screen_map.set_num_y_sections(0);
					}
					int plot_number = 0;
					for (int ki = 0; ki < keyVector.size() && !clearing_plots; ki++) {
						if (plots_to_show > max_plots_to_show && max_plots_to_show > 0 && (ki / max_plots_to_show) != plot_group_number) {
							continue;
						}
						PlotData plot_data = keyVector.elementAt(ki);
						try {
							synchronized (plot_data) {
								plot_data.painting = true;
								if (plotter_num > 0 && debug_on) {
									System.out.println("Repainting PlotGraph plot_data = " + plot_data);
									System.out.println("Repainting PlotGraph plot_data.show = " + plot_data.show);
								}
								if (null != plot_data.show && plotter_num >= 0) {
									if (plot_data.show.length > plotter_num) {
										if (!plot_data.show[plotter_num] || plot_data.array_type != array_mode) {
											continue;
										}
									}
								}
								if (plot_data.line_width > 0) {
									line_width = plot_data.line_width;
								}
								line_style = plot_data.line_style;
								PlotPoint last_point = null;
								int j = 0;
								if (plotter_num > 0 && debug_on) {
									//System.out.println("Repainting PlotGraph orig_scroll_x = "+orig_scroll_x);
									//System.out.println("Repainting PlotGraph orig_scroll_y = "+orig_scroll_y);
									System.out.println("Repainting PlotGraph cur_screen_map.get_screen_width() = " + cur_screen_map.get_screen_width());
									System.out.println("Repainting PlotGraph cur_screen_map.get_screen_height() = " + cur_screen_map.get_screen_height());
								}
								int v_size = plot_data.v_size();
								double this_plot_section_y_min = Double.MAX_VALUE;
								double this_plot_section_y_max = -Double.MAX_VALUE;
								last_plots_to_show = plots_to_show;
								if (s_mode) {
									for (j = 0; j < v_size && j < plot_data.current_size && !clearing_plots; j++) {
										//int v_index = (j + plot_data.v_offset)%v_size;
										PlotPoint current_point = plot_data.getPlotPointAt(j);
										current_point.x = cur_screen_map.get_screen_x(current_point.orig_x);
										if (current_point.x >= 0 &&
												current_point.x <= width) {
											if (current_point.orig_y > this_plot_section_y_max) {
												this_plot_section_y_max = current_point.orig_y;
											}
											if (current_point.orig_y < this_plot_section_y_min) {
												this_plot_section_y_min = current_point.orig_y;
											}
										}
									}
									cur_screen_map.set_y_section_show_area(plot_number, this_plot_section_y_min, this_plot_section_y_max);
								}

								if (plot_data.mark_points && plot_data.current_size > point_size_limit && m_color) {
									g.setColor(plot_data.point_color);
								}
								for (j = 0; j < v_size && j < plot_data.current_size && !clearing_plots; j++) {
									int v_index = (j + plot_data.v_offset) % v_size;
									PlotPoint current_point = plot_data.getPlotPointAt(v_index);
									if (null == current_point) {
										continue;
									}
									current_point.x = cur_screen_map.get_screen_x(current_point.orig_x);
									current_point.y = cur_screen_map.get_section_screen_y(plot_number, current_point.orig_y);
									//							System.out.println("p="+current_point);
									if (plotter_num > 0 && debug_on) {
										System.out.println("Repainting PlotGraph current_point.x = " + current_point.x);
										System.out.println("Repainting PlotGraph current_point.y = " + current_point.y);
									}

									if ((current_point.x < 0 || current_point.x > width) && !xy_mode) {
										last_point = current_point;
										continue;
									}
									if (plot_data.current_size > point_size_limit || !plot_data.mark_points) {
										g.drawRect(current_point.x,
												current_point.y,
												0, 0);
									} else {
										if (m_color) {
											g.setColor(plot_data.line_color);
										}
										if (null != last_point && show_lines && c_mode) {

											if (last_point.x < 0 || last_point.x > width) {
												last_point = current_point;
												continue;
											}
											g.drawLine(last_point.x, last_point.y,
													current_point.x, current_point.y);
										}
										if (plot_data.mark_points) {
											if (m_color) {
												g.setColor(plot_data.point_color);
											}
											int fr_x = current_point.x - 2;
											int fr_y = current_point.y - 2;
											g.fillRect(fr_x,
													fr_y,
													4, 4);
										}
										if (label_points) {

											if (null != last_point) {
												if (Math.abs(current_point.x - last_point.x) < 30 && Math.abs(current_point.y - last_point.y) < 10) {
													last_point = current_point;
													continue;
												}
												if (label_string != null) {
													if (null == font) {
														font = g.getFont();
														fm = g.getFontMetrics(font);
													}
													if (null != fm) {
														if (Math.abs(current_point.x - last_point.x) < fm.stringWidth(label_string) && Math.abs(current_point.y - last_point.y) < fm.getHeight()) {
															last_point = current_point;
															continue;
														}
													}
												}
											}
											label_string = "(" + current_point.orig_x + "," + current_point.orig_y + ")";
											g.drawString(label_string, current_point.x + 5, current_point.y - 5);
										}
										last_point = current_point;
									}
								}
								if (s_mode) {
									int this_plot_section_y_offset = cur_screen_map.get_section_screen_y_line_pose(plot_number);
									int top_label_y_pos = this_plot_section_y_offset - (height / plots_to_show) + 20;
									int bottom_label_y_pos = this_plot_section_y_offset - 15;

//DebugPrint2("plot_number="+plot_number+", this_plot_section_y_offset="+this_plot_section_y_offset+", top_label_y_pos="+top_label_y_pos+", height="+height+", plots_to_show="+plots_to_show+", bottom_label_y_pos="+bottom_label_y_pos+", plot_data.short_name="+plot_data.short_name);
									String slabel = "";
									if (m_color && axis_color != null) {
										g.setColor(axis_color);
									}
									if (show_key && !s_mode) {
										slabel = "> " + this_plot_section_y_min;
									} else {
										String tail = "";
										if (null != short_extra_sh_str) {
											tail += short_extra_sh_str;
										}
										if (null != plot_data.add_to_short_key) {
											tail += plot_data.add_to_short_key;
										}
										slabel = "> " + this_plot_section_y_min + " : " + plot_data.short_name + tail;
									}
									if (height / plots_to_show > 50 || (max_plots_to_show > 1 && height / max_plots_to_show > 50)) {
										g.drawString("< " + this_plot_section_y_max, 5, top_label_y_pos);
									} else {
										slabel += " < " + this_plot_section_y_max;
									}
									g.drawString(slabel, 5, bottom_label_y_pos);
									g.drawLine(0,
											this_plot_section_y_offset,
											width,
											this_plot_section_y_offset);
								}
							}
							plot_number++;
						} catch (Exception e) {
							e.printStackTrace();
						}
						plot_data.painting = false;
					}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	public void SetOuterArea(double min_x, double max_x, double min_y, double max_y) {
		if (array_mode) {
			array_mode_screen_map.set_abs_x_min(min_x);
			array_mode_screen_map.set_abs_x_max(max_x);
			array_mode_screen_map.set_abs_y_max(max_y);
			array_mode_screen_map.set_abs_y_min(min_y);
			if (e_mode) {
				array_mode_screen_map.equalizeAxis();
			}
		} else {
			screen_map.set_abs_x_min(min_x);
			screen_map.set_abs_x_max(max_x);
			screen_map.set_abs_y_max(max_y);
			screen_map.set_abs_y_min(min_y);
			if (e_mode) {
				screen_map.equalizeAxis();
			}
		}
	}

	public void SetInnerArea(double min_x, double max_x, double min_y, double max_y) {
		if (array_mode) {
			array_mode_screen_map.set_x_show_area(min_x, max_x);
			array_mode_screen_map.set_y_show_area(min_y, max_y);
			if (e_mode) {
				array_mode_screen_map.equalizeAxis();
			}
		} else {
			screen_map.set_x_show_area(min_x, max_x);
			screen_map.set_y_show_area(min_y, max_y);
			if (e_mode) {
				screen_map.equalizeAxis();
			}
		}
	}
	private int plot_group_number = 0;

	public void set_plot_group_number(int _plot_group_number) {
		this.plot_group_number = _plot_group_number;
	}
	/**
	 * Holds value of property max_plots_to_show.
	 */
	private int max_plots_to_show;

	/**
	 * Getter for property max_plots_to_show.
	 * @return Value of property max_plots_to_show.
	 */
	public int getMax_plots_to_show() {
		return this.max_plots_to_show;
	}

	/**
	 * Setter for property max_plots_to_show.
	 * @param max_plots_to_show New value of property max_plots_to_show.
	 */
	public void setMax_plots_to_show(int max_plots_to_show) {
		this.max_plots_to_show = max_plots_to_show;
	}
}
