/*
The NIST RCS (Real-time Control Systems)
library is public domain software, however it is preferred
that the following disclaimers be attached.

Software Copywrite/Warranty Disclaimer

This software was developed at the National Institute of Standards and
Technology by employees of the Federal Government in the course of their
official duties. Pursuant to title 17 Section 105 of the United States
Code this software is not subject to copyright protection and is in the
public domain. NIST Real-Time Control System software is an experimental
system. NIST assumes no responsibility whatsoever for its use by other
parties, and makes no guarantees, expressed or implied, about its
quality, reliability, or any other characteristic. We would appreciate
acknowledgement if the software is used. This software can be
redistributed and/or modified freely provided that any derivative works
bear some notice that they are derived from it, and any modified
versions bear some notice that they have been modified. 

 */
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package rcs.nml;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.net.URL;
import java.net.URLConnection;
import java.util.Arrays;

/**
 * Class that allows the reading of packed files directly without using
 * an NMLConnection.
 * 
 * @author Will Shackleford <shackle@nist.gov>
 */
public class PackedFileReader {

	private PackedFormatConverter converter = null;

	public PackedFileReader(NMLMessageDictionary _dict, boolean _l64_mode) {
		converter = new PackedFormatConverter(_l64_mode);
		converter.SetMessageDictionary(_dict);
	}

	public NMLmsg ReadFile(final String fileName) {
		File f = new File(fileName);
		if(f.exists()) {
			return ReadFile(f);
		}
		else if(fileName.startsWith("http://"))
		{
			try
			{
				URL url_object= new URL(fileName);
				URLConnection url_connection = url_object.openConnection();
				InputStream is = url_connection.getInputStream();
				int bytes_available=0;
				int bytes_read=0;
				byte old_raw_data[] = null;
				byte raw_data[] = null;
				int read_ret=-1;
				bytes_available=is.available();
				final int content_length = url_connection.getContentLength();
				raw_data = new byte[content_length];
				read_ret = is.read(raw_data);
				if(read_ret > 0) {
					bytes_read += read_ret;
				}
				while(bytes_read < content_length)
				{
					read_ret = is.read(raw_data, bytes_read, raw_data.length - bytes_read);
					if(read_ret > 0)
					{
						bytes_read += read_ret;
					}
					else
					{
						break;
					}
				}
				is.close();
				is=null;
				url_connection=null;
				url_object=null;
				return converter.convertRawDataToMsg(raw_data, 0, raw_data.length);
			}
			catch(Exception e)
			{
				e.printStackTrace();
			}
		}
		return null;
	}

	public NMLmsg ReadFile(final File curFile) {
		FileInputStream fis = null;
		byte raw_data[] = null;
		NMLmsg tmpMsg = null;
		try {
			fis = new FileInputStream(curFile);
			raw_data = new byte[fis.available()];
			fis.read(raw_data);
			fis.close();
			fis = null;
			tmpMsg = converter.convertRawDataToMsg(raw_data, 0, raw_data.length);
		} catch (Exception except) {
			except.printStackTrace(rcs.nml.debugInfo.debugPrintStream);
			rcs.nml.debugInfo.debugPrintStream.println("Can't ReadFile(" + curFile + ")");
		} finally {
			if (null != fis) {
				try {
					fis.close();
				} catch (Exception ex2) {
					ex2.printStackTrace();
				}
				fis = null;
			}
			raw_data = null;
		}
		return tmpMsg;
	}

	public NMLmsg ReadFile(final File curFile, int max_len) {
		FileInputStream fis = null;
		byte raw_data[] = null;
		NMLmsg tmpMsg = null;
		try {
			boolean hide_errors_orig = converter.hide_errors;
			fis = new FileInputStream(curFile);
			int len = fis.available();
			if (len > max_len) {
				converter.hide_errors = true;
				len = max_len;
			}
			raw_data = new byte[len];
			fis.read(raw_data);
			fis.close();
			fis = null;
			tmpMsg = converter.convertRawDataToMsg(raw_data, 0, raw_data.length);
			converter.hide_errors = hide_errors_orig;
		} catch (Exception except) {
			except.printStackTrace(rcs.nml.debugInfo.debugPrintStream);
			rcs.nml.debugInfo.debugPrintStream.println("Can't ReadFile(" + curFile + ")");
		} finally {
			if (null != fis) {
				try {
					fis.close();
				} catch (Exception ex2) {
					ex2.printStackTrace();
				}
				fis = null;
			}
			raw_data = null;
		}
		return tmpMsg;
	}

	public String convertMsgToString(final NMLmsg inMsg) {
		return converter.convertMsgToString(inMsg);
	}
}
