/* 
The NIST RCS (Real-time Control Systems) 
 library is public domain software, however it is preferred
 that the following disclaimers be attached.

Software Copywrite/Warranty Disclaimer

   This software was developed at the National Institute of Standards and
Technology by employees of the Federal Government in the course of their
official duties. Pursuant to title 17 Section 105 of the United States
Code this software is not subject to copyright protection and is in the
public domain. NIST Real-Time Control System software is an experimental
system. NIST assumes no responsibility whatsoever for its use by other
parties, and makes no guarantees, expressed or implied, about its
quality, reliability, or any other characteristic. We would appreciate
acknowledgement if the software is used. This software can be
redistributed and/or modified freely provided that any derivative works
bear some notice that they are derived from it, and any modified
versions bear some notice that they have been modified.



*/ 


#include "nml_test_format.hh"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>

static bool sigint_occured=false;

static void my_sigint_handler(int /*sig*/)
{
  sigint_occured=true;
}

int 
main(int argc, char **argv)
{
  int repeat_count;
  double repeat_delay;
  NML *nml = 0;
  repeat_count=0;
  repeat_delay=1.0;
  set_rcs_print_destination(RCS_PRINT_TO_STDERR);

  if(argc < 5)
    {
      fprintf(stderr,"usage : bufname procname configfile  lastvar\n");
      exit(127);
    }
  if(argc >= 6)
    {
      repeat_count=strtol(argv[5],0,0);
    }
  if(argc >= 7)
    {
      repeat_delay =strtod(argv[6],0);
    }

  nml = new NML(nml_test_format,argv[1],argv[2],argv[3]);
  if(0 == nml)
    {
      exit(126);
    }
  if(!nml->valid())
    {
      fprintf(stderr,"nml->valid() check failed.\n");
      delete nml;
      exit(125);
    }
  nml_start();

  int t;
  double max_time_diff = 0.0;
  double total_time_diff=0.0;
  int num_reads=0;
  int new_reads=0;
  int expected_last_var=strtol(argv[4],0,0);
  double start_read_time=etime();
  nml->cms->do_not_print_timeout_errors=true;
  t = nml->read();
  double end_time=etime();
  double time_diff = end_time-start_read_time;
  max_time_diff=time_diff;
  total_time_diff += time_diff;
  num_reads++;
  if(t>0)
    {
      new_reads++;
    }
  printf("time to read=%f\n",time_diff);
  switch(t)
    {
    case TEST_MESSAGE_TYPE:
      {
	TEST_MESSAGE *tst_msg = (TEST_MESSAGE *) nml->get_address();
	if(0 == tst_msg)
	  {
	    fprintf(stderr,"nml->get_address() returned NULL\n");
	    delete nml;
	    exit(123);
	  }
	if(tst_msg->first_count != tst_msg->last_count)
	  {
	    fprintf(stderr,"tst_msg->first_count(%ld) != tst_msg->last_count(%ld)\n", tst_msg->first_count, tst_msg->last_count);
	    delete nml;
	    exit(122);
	  }
	if(tst_msg->lastvar != expected_last_var)
	  {
	    fprintf(stderr,"expected_last_var(%d) != tst_msg->last_var(%ld)\n",
		    expected_last_var, tst_msg->lastvar);
	    delete nml;
	    exit(121);
	  }
      }
      break;

    case SIMPLER_MSG_TYPE:
      {
	SIMPLER_MSG *smsg = (SIMPLER_MSG *) nml->get_address();
	if(0 == smsg)
	  {
	    fprintf(stderr,"nml->get_address() returned NULL\n");
	    delete nml;
	    exit(120);
	  }
	if(smsg->lastvar != expected_last_var)
	  {
	    fprintf(stderr,"expected_last_var(%d) != smsg->last_var(%ld)\n",
		    expected_last_var, smsg->lastvar);
	    delete nml;
	    exit(119);
	  }
      }
      break;

    case 0:
      if(repeat_count >= 0 && repeat_count <= 1)
	{
	  fprintf(stderr,"nml->read() returned %d when %d was expected.\n", t,TEST_MESSAGE_TYPE);
	  delete nml;
	  exit(118);
	}
      break;

    default:
      fprintf(stderr,"nml->read() returned %d when %d was expected.\n", t,TEST_MESSAGE_TYPE);
      delete nml;
      exit(118);
    }
  int i=0;
  void (*old_handler)(int)=0;
  if(repeat_count != 0)
    {
      old_handler = signal(SIGINT,my_sigint_handler);
      if(repeat_delay > 0)
	{
	  nml->setup_subscription(repeat_delay);
	}
    }
  for(i=0;
      ((i < repeat_count || repeat_count < 0) && !sigint_occured) ; i++)
    {      
      if(repeat_delay > 0)
	{
	  esleep(repeat_delay);
	}
      start_read_time=etime();
      t = nml->read();
      end_time=etime();
      time_diff = end_time-start_read_time;
      if(time_diff > max_time_diff)
	{
	  max_time_diff=time_diff;
	}
      total_time_diff += time_diff;
      num_reads++;
      if(t>0)
	{
	  new_reads++;
	}
      printf("time to read=%f (max=%f,#=%d,avg=%f,new=%d,percent_new=%f%%)\n",
	     time_diff,
	     max_time_diff,
	     num_reads,
	     (total_time_diff/num_reads),
	     new_reads,
	     ((double) (new_reads*100.0)/num_reads));
      rcs_print("nml->read() returned %d\n",t);
      switch(t)
	{
	case TEST_MESSAGE_TYPE:
	  {
	    TEST_MESSAGE *tst_msg2 = (TEST_MESSAGE *) nml->get_address();
	    rcs_print("tst_msg2->i=%d\n",tst_msg2->i);
	    if(tst_msg2->first_count != tst_msg2->last_count)
	      {
		fprintf(stderr,"tst_msg->first_count(%ld) != tst_msg->last_count(%ld)\n", tst_msg2->first_count, tst_msg2->last_count);
		delete nml;
		exit(117);
	      }
	    if(tst_msg2->lastvar != expected_last_var)
	      {
		fprintf(stderr,"expected_last_var(%d) != tst_msg2->last_var(%ld)\n",
			expected_last_var, tst_msg2->lastvar);
		signal(SIGINT,old_handler);
		delete nml;
		exit(116);
	      }
	  }
	  break;
	  
	case 0:
	  break;
	  
	case -1:
	default:
	  fprintf(stderr,"Bad return value from read.\n");
	  signal(SIGINT,old_handler);
	  delete nml;
	  exit(115);
	}
    }
  if(repeat_count != 0)
    {
      signal(SIGINT,old_handler);
    }
  delete nml;
  nml_cleanup();
  exit(0);
}
